import numpy as np
import pytest
from mlx import nn
from sklearn.datasets import make_regression

from sklx.regressor import NueralNetworkRegressor


class TestNueralNetRegressor:
    @pytest.fixture
    @staticmethod
    def nerual_network_regressor():
        class RegressorModule(nn.Module):
            def __init__(
                self,
                num_units=10,
                nonlin=nn.relu,
            ):
                super().__init__()
                self.num_units = num_units
                self.layers = [
                    nn.Linear(20, num_units),
                    nn.Linear(num_units, 10),
                    nn.Linear(10, 1),
                ]
                self.dense0 = nn.Linear(20, num_units)
                self.nonlin = nonlin
                self.dense1 = nn.Linear(num_units, 10)
                self.output = nn.Linear(10, 1)

            def forward(self, X, **kwargs):
                X = self.nonlin(self.dense0(X))
                X = nn.relu(self.dense1(X))
                X = self.output(X)
                return X

            def __call__(self, X, **kwargs):
                for _, layer in enumerate(self.layers):
                    X = layer(X)
                return X

        net = NueralNetworkRegressor(RegressorModule, max_epochs=10, lr=0.1)
        return net

    def test_regressor(self, nerual_network_regressor):
        """
        Test to make sure the basic functionality of the regressor works.
        """
        X_regr, y_regr = make_regression(1000, 20, n_informative=10, random_state=0)
        X_regr = X_regr.astype(np.float32)
        y_regr = y_regr.astype(np.float32) / 100
        y_regr = y_regr.reshape(-1, 1)

        nerual_network_regressor.fit(X_regr, y_regr)
        nerual_network_regressor.predict(X_regr)



from typing import Callable

import mlx.optimizers.optimizers as optimizers
import numpy as np
from mlx import nn
from sklearn.base import ClassifierMixin

from sklx.net import NeuralNet


class NeuralNetworkClassifier(ClassifierMixin, NeuralNet):
    """
    Implementation of the NerualNet base class. See `sklx.net.NerualNet` for detailed documentation.
    """

    module = None
    max_epochs = 10
    lr = 0.1
    batch_size = 10
    optimizer = optimizers.SGD

    def __init__(
        self,
        module: nn.Module,
        max_epochs: float,
        lr: float,
        criterion: Callable,
        classes=None,
    ) -> None:
        super().__init__(
            module=module, criterion=criterion, lr=lr, max_epochs=max_epochs
        )
        self.classes = classes

    @property
    def classes_(self):
        return np.array(self.classes)



from mlx.nn import losses
from sklearn.base import RegressorMixin

from sklx.net import NeuralNet


class NueralNetworkRegressor(RegressorMixin, NeuralNet):
    """
    Implmentation of the NueralNet class. See `sklx.NeuralNet` for more details.
    """

    def __init__(self, module, *args, criterion=losses.mse_loss, **kwargs):
        super().__init__(module, *args, criterion=criterion, **kwargs)



import numpy as np
import pytest
from mlx import nn
from mlx.nn import losses
from sklearn.datasets import make_classification
from sklearn.model_selection import GridSearchCV
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler

from sklx.classifier import NeuralNetworkClassifier


class TestNeuralNetworkClassifier:
    @pytest.fixture
    @staticmethod
    def nerual_network_classifier():
        class MyModule(nn.Module):
            def __init__(self, num_units=10, nonlin=nn.ReLU()):
                super().__init__()
                self.layers = [
                    nn.Linear(20, num_units),
                    nonlin,
                    nn.Dropout(0.5),
                    nn.Linear(num_units, num_units),
                    nn.Linear(num_units, 2),
                    nn.LogSoftmax(),
                ]

            def __call__(self, X, **kwargs):
                for _, layer in enumerate(self.layers):
                    X = layer(X)
                return X

        net = NeuralNetworkClassifier(
            MyModule, max_epochs=10, lr=0.1, criterion=losses.nll_loss
        )

        return net

    def test_neural_network_classifier(self, nerual_network_classifier):
        """
        This is just a simple test to make sure the basic usage works.
        """
        X, y = make_classification(1000, 20, n_informative=10, random_state=0)
        X = X.astype(np.float32)
        y = y.astype(np.int64)
        nerual_network_classifier.fit(X, y)
        nerual_network_classifier.predict_proba(X)

    def test_sklearn_pipeline_support(self, nerual_network_classifier):
        """
        Test to make sure that using a model in a Sklearn pipeline works.
        """
        X, y = make_classification(1000, 20, n_informative=10, random_state=0)
        X = X.astype(np.float32)
        y = y.astype(np.int64)

        pipe = Pipeline(
            [
                ("scale", StandardScaler()),
                ("net", nerual_network_classifier),
            ]
        )

        pipe.fit(X, y)
        pipe.predict_proba(X)
        assert pipe.score(X, y) > 0.6

    def test_sklearn_grid_search_support(self, nerual_network_classifier):
        """
        Test to make sure that using a model with Sklearn Grid Search works.
        """
        X, y = make_classification(1000, 20, n_informative=10, random_state=0)
        X = X.astype(np.float32)
        y = y.astype(np.int64)

        params = {
            "lr": [0.01, 0.03],
            "max_epochs": [10, 30],
            "module__num_units": [10, 30],
        }
        gs = GridSearchCV(
            nerual_network_classifier,
            params,
            refit=False,
            cv=3,
            scoring="accuracy",
            verbose=2,
        )
        gs.fit(X, y)
        assert gs.best_score_ > 0.75
